/*
TODO:
- check if the "int" type for the oss parameter is appropriate for function getPressure;
- check if the ac5_sur2 variable can be eliminated in getTemperature function;

*/

#include "BMP085.h"

void BMP085::init(){
  ac1 = (long) readShort(0xAA);
  ac2 = (long) readShort(0xAC);  
  ac3 = readShort(0xAE);
  ac4 = (unsigned short) readShort(0xB0);
  ac5 = (unsigned short) readShort(0xB2);
  ac6 = (unsigned short) readShort(0xB4); 
  b1 = readShort(0xB6);
  b2 = readShort(0xB8);
  mb = readShort(0xBA);
  mc = readShort(0xBC);
  md = readShort(0xBE);  
};

unsigned long BMP085::getTemperature(){
  return getMeasures(0,0);
};

unsigned long BMP085::getPressure(int oss){
  return getMeasures(1,oss);
};

unsigned long BMP085::getPressure(){
  return getMeasures(1,0);
};

unsigned long BMP085::getMeasures(unsigned char type, int oss){
//  Wire.beginTransmission(BMP085_I2C_ADDRESS); // transmit to device #4
//  Wire.write(0xF4);
//  Wire.write(0x2E);
//  Wire.endTransmission();
  
  unsigned char data[3];
  data[0]=0xF4;
  data[1]=0x2E;
  data[2]=0;
  write(&data[0]);
  
  delay(5);
  
  long ut = (unsigned short) readShort(0xF6);
  //long ac5_sur2 = ac5 / 2L;
  //long x1 = (ut - ac6) * ac5_sur2 / 16384L;
  long x1 = (ut - ac6) * ac5 / 32768L;
  long x2 = mc * 2048L / (x1 + md);
  long b5 = x1 + x2;
  if (type==0) return (b5 + 8L) / 16L;

  int tempo = 0;
  
//  Wire.beginTransmission(BMP085_I2C_ADDRESS);  
//  Wire.write(0xF4);
//  switch (oss) {
//    case 0:
//      Wire.write(0x34);
//      tempo = 5;
//      break;
//    case 1:
//      Wire.write(0x74);
//      tempo = 8;
//      break;
//    case 2:
//      Wire.write(0xB4);
//      tempo = 14;
//      break;
//    case 3:
//      Wire.write(0xF4);
//      tempo = 26;
//      break;
//  }
//  Wire.endTransmission();
  
  switch (oss) {
    case 0:
      data[1] = 0x34;
      tempo = 5;
      break;
    case 1:
      data[1] = 0x74;
      tempo = 8;
      break;
    case 2:
      data[1] = 0xB4;
      tempo = 14;
      break;
    default:
      data[1] = 0xF4;
      tempo = 26;
      break;
  }
  
  write(&data[0]);
  
  delay(tempo);  

  long up = readLong(0xF6);
  up = up >> (8 - oss);

  long b6 = b5 - 4000;
  x1 = (b2 * (b6 * b6 / 4096)) / 2048;
  x2 = ac2 * b6 / 2048;
  long x3 = x1 + x2;
  long b3 = (((ac1*4+x3)<<oss) + 2)/4;
  x1 = ac3 * b6 / 8192;
  x2 = (b1 * (b6 * b6 / 4096)) / 65536;
  x3 = ((x1 + x2) + 2) / 4;
  unsigned long b4 = ac4 * (unsigned long)(x3 + 32768) / 32768;
  long b7 = ((unsigned long)up - b3) * (50000 >> oss);
  long p;
  if (b7< 0x80000000) { p = (b7 *2) / b4;}
  else { p = (b7/b4) * 2;}
  x1 = (p/256)*(p/256);
  x1 = (x1 * 3038) / 65536;
  x2 = (-7357 * p) / 65536;
  p = p + (x1+x2+3791)/16;

  return p;
};

short BMP085::readShort(unsigned char reg){
//  Wire.beginTransmission(BMP085_I2C_ADDRESS); // Begin a write transmission with the BMP085
//  Wire.write(reg);  // Writes the register which is to be read in the BMP085
//  Wire.endTransmission();  // Closes the write transmission
  
  unsigned char data[2];
  data[0]=reg;
  data[1]=0;
  write(&data[0]);
  
  Wire.requestFrom(BMP085_I2C_ADDRESS, 2);    // request 2 bytes from the BMP085
  short val = 0;
  while (Wire.available())    
  {
    val = val << 8;
    unsigned char c = Wire.read(); 
    val = val | c;
  }
  return val;  
};

long BMP085::readLong(unsigned char reg)
{
//  Wire.beginTransmission(BMP085_I2C_ADDRESS);
//  Wire.write(reg);
//  Wire.endTransmission();

  unsigned char data[2];
  data[0]=reg;
  data[1]=0;
  write(&data[0]);
  
  Wire.requestFrom(BMP085_I2C_ADDRESS, 3);    // request 3 bytes from the BMP085
  long val = 0;
  while (Wire.available())    
  {
    val = val << 8;
    unsigned char c = Wire.read(); 
    val = val | c;
  }
  return val;
};

void BMP085::write(unsigned char * ptr) {
  Wire.beginTransmission(BMP085_I2C_ADDRESS);
  while (*ptr!=0) {
    Wire.write(*ptr);
    ptr++;
  }
  Wire.endTransmission();
}
